local _, class = UnitClass("player")
local texture = [=[Interface\AddOns\oUF_coree\media\Glamour2a]=]
local texture2 = [=[Interface\AddOns\oUF_coree\media\tex]=]
local fontn = "Interface\\AddOns\\oUF_coree\\media\\accid.ttf"
local backdrop = {
  bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
  edgeFile = "", edgeSize = 16,
  insets = {left = -1, right = -1, top = -1, bottom = -1},
}

local width, height = 216, 20
local targetshort = false -- true = short hpvalue for target
local cbar = true -- true = oUF castbar 
local cbarsafe = true -- true = latency zone for the castbar
local partyraid = true -- true = show party in raid
local ppets = true -- true = show party pets
local inCombat = false -- true = show icon when entering combat

local colors = setmetatable({
  power = setmetatable({
    ["MANA"] = {0, 144/255, 1},
    ["AMMOSLOT"] = {0.8, 0.6, 0},
    ["FUEL"] = {0, 0.55, 0.5},
    ["POWER_TYPE_STEAM"] = {0.55, 0.57, 0.61},
    ["POWER_TYPE_PYRITE"] = {0.60, 0.09, 0.17},
  }, {__index = oUF.colors.power}),
  reaction = setmetatable({  
    [1] = {1, 0, 0},
    [2] = {1, 0, 0},
    [3] = {0.75, 0.27,0},
    [4] = {1.0, 0.96, 0},
    [5] = {0, 1, 0.1},
    [6] = {0, 1, 0.1},
    [7] = {0, 1, 0.1},
    [8] = {0, 1, 0.1},
    }, {__index = oUF.colors.reaction}),
  runes = setmetatable({
    [1] = {0.8, 0, 0},
    [3] = {0, 0.4, 0.7},
    [4] = {0.8, 0.8, 0.8}
  }, {__index = oUF.colors.runes})
}, {__index = oUF.colors})

local function Hex(r, g, b)
  if type(r) == "table" then
    if r.r then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
  end
  return string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
end

oUF.Tags["[coreediff]"]  = function(u) local l = UnitLevel(u); return Hex(GetQuestDifficultyColor((l > 0) and l or 99)) end
oUF.TagEvents["[coreeselect]"] = oUF.TagEvents["[pvp]"]
oUF.Tags["[coreeselect]"] = function(u) local c = colors.reaction[UnitReaction(u, "player")] return Hex(c[1],c[2],c[3]) end
oUF.Tags["[coreecolor]"] = function(u)
  return   ((UnitIsFriend("player", "target") and UnitPlayerControlled("target")) or not UnitIsConnected(u)) and oUF.Tags["[raidcolor]"](u) or
      (not UnitIsFriend("player", "target") and UnitPlayerControlled("target")) and oUF.Tags["[coreeselect]"](u) or 
      oUF.Tags["[coreediff]"](u)
end
oUF.TagEvents["[coreeclassi]"] = oUF.TagEvents["[classification]"]
oUF.Tags["[coreeclassi]"] = function(u)
  local c = UnitClassification(u);
  return c == "rare" and "Rare" or c == "rareelite" and "+ Rare" or c == "elite" and "+" or c == "worldboss" and "Boss" or ""
end
oUF.TagEvents["[coreeafk]"] = "PLAYER_FLAGS_CHANGED"
oUF.Tags["[coreeafk]"] = function(u) return UnitIsAFK(u) and "<AFK> " or "" end
oUF.TagEvents["[coreehpp]"] = oUF.TagEvents["[curhp]"]
oUF.Tags["[coreehpp]"] = function(u)
  return  (not UnitIsConnected(u)) and oUF.Tags["[offline]"](u) or
      UnitIsDead(u) and "Dead" or
      UnitIsGhost(u) and "Ghost" or oUF.Tags["[curhp]"](u)
end
oUF.TagEvents["[coreehp]"] = "UNIT_HEALTH UNIT_MANA UNIT_RAGE UNIT_ENERGY UNIT_RUNIC_POWER UNIT_MAXMANA UNIT_MAXRAGE UNIT_MAXENERGY UNIT_DISPLAYPOWER UNIT_MAXRUNIC_POWER PLAYER_REGEN_ENABLED PLAYER_REGEN_DISABLED"
oUF.Tags["[coreehp]"] = function(u)
  local min, max, ma = UnitHealth(u), UnitHealthMax(u), UnitMana("player")
  local combat = UnitAffectingCombat("player")
  return   (not UnitIsConnected(u) and "Offline") or (UnitIsDead(u) and "Dead") or (UnitIsGhost(u) and "Ghost")
      or (u == "target" and targetshort) and ShortHp(min)
      or (u == "target" and not targetshort) and min
      or (u == "player" and (min~=max or ma > 0 or combat == 1)) and min
      or (u == "player" and ma == 0) and "" or ""
end
oUF.TagEvents["[coreehp2]"] = oUF.TagEvents["[perpp]"]
oUF.Tags["[coreehp2]"] = function(u)
  local min, max = UnitHealth(u), UnitHealthMax(u)
  local d = floor(min/max*100)
  return   (not UnitIsConnected(u) or UnitIsDead(u) or UnitIsGhost(u)) and ""
      or (d < 100 ) and d.."%"
      or ""
end
oUF.TagEvents["[coreepp]"] = oUF.TagEvents["[curpp]"]
oUF.Tags["[coreepp]"] = function(u)
  local min = UnitMana(u)
  local _, ptype = UnitPowerType(u)
  local color = colors.power[ptype]
  return   (not UnitIsPlayer(u) or not UnitIsConnected(u) or UnitIsDeadOrGhost(u) or min == 0) and "" 
      or format(min .. "|cff%02x%02x%02x .|r", color[1]*255, color[2]*255, color[3]*255)
end

oUF.TagEvents["[coreenamev]"] = oUF.TagEvents["[name]"]
oUF.Tags["[coreenamev]"] = function(u, r, realUnit)
  return UnitName(realUnit or u or r)
end

local function menu(self)
  local unit = string.gsub(self.unit, "(.)", string.upper, 1)
  if(_G[unit.."FrameDropDown"]) then
    ToggleDropDownMenu(1, nil, _G[unit.."FrameDropDown"], "cursor")
  end
end

local function ShortHp(value)
  if(value >= 1e6) then
    return string.format("%.1fm", value / 1e6)
  elseif(value >= 1e4) then
    return string.format("%.1fk", value / 1e3)
  else
    return value
  end
end

local function SetFontString(parent, fontName, fontHeight, fontStyle)
  local fs = parent:CreateFontString(nil, "OVERLAY")
  fs:SetFont(fontName, fontHeight, fontStyle)
  fs:SetJustifyH("LEFT")
  fs:SetShadowColor(0,0,0)
  fs:SetShadowOffset(1, -1)
  return fs
end

local function updateRIcon(self, event)
  local index = GetRaidTargetIndex(self.unit)
  if(index) then
    self.RIcon:SetText(ICON_LIST[index].."22|t")
  else
    self.RIcon:SetText()
  end
end

local function playerVehicle(self, event, unit)
  if self.unit ~= unit then return end
  if event == "UNIT_ENTERED_VEHICLE" then
    self.Info:Show()
  elseif event == "UNIT_EXITED_VEHICLE" then
    self.Info:Hide()
  end
end

oUF.TagEvents['[druidpower]'] = 'UNIT_MANA UPDATE_SHAPESHIFT_FORM'
oUF.Tags['[druidpower]'] = function(unit)
  local min, max = UnitPower(unit, 0), UnitPowerMax(unit, 0)
  return UnitPowerType(unit) ~= 0 and format('|cff0090ff%d - %d%%|r', min, math.floor(min / max * 100))
end

local function updateDruidPower(self, event, unit)
  if(unit and unit ~= self.unit) then return end
  local bar = self.DruidPower

  local mana = UnitPowerType('player') == 0
  local min, max = UnitPower('player', mana and 3 or 0), UnitPowerMax('player', mana and 3 or 0)

  bar:SetStatusBarColor(unpack(colors.power[mana and 'ENERGY' or 'MANA']))
  bar:SetMinMaxValues(0, max)
  bar:SetValue(min)
  bar:SetAlpha(min ~= max and 1 or 0)
end

local function castbarTime(self, duration)
  if(self.channeling) then
    self.Time:SetFormattedText('%.1f ', duration)
  elseif(self.casting) then
    self.Time:SetFormattedText('%.1f ', self.max - duration)
  end
end

local function PostUpdateExperience(self, event, unit, bar)
  local bar = self.Experience
  if(unit == "player" and GetXPExhaustion()) then
    bar:SetStatusBarColor(0.0, 0.39, 0.88, 1.0)
  else
    bar:SetStatusBarColor(0.58, 0.0, 0.55, 1.0)
  end
end

local function createAura(icons, button)
  local pcolor = UnitIsPlayer("player") and RAID_CLASS_COLORS[select(2, UnitClass("player"))]
  icons.showDebuffType = true
  button.cd:SetReverse()
  button.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
  button.overlay:SetTexture([=[Interface\AddOns\oUF_coree\media\border]=])
  button.overlay:SetTexCoord(0.0, 1.0, 0.0, 1.0)
  if (pcolor) then
    button.overlay.Hide = function(self) self:SetVertexColor(pcolor.r, pcolor.g, pcolor.b) end
  else 
    button.overlay.Hide = function(self) self:SetVertexColor(.25, .25, .25) end
  end
  button:SetScript('OnMouseUp', onMouseUp)
end

local PostUpdateAuraIcon
do
  local playerUnits = {
    player = true,
    pet = true,
    vehicle = true,
  }
 
  PostUpdateAuraIcon = function(self, icons, unit, icon, index, offset, filter, isDebuff)
    local texture = icon.icon
	local dur = icon.cd
	if isDebuff then
      if(playerUnits[icon.owner] or UnitIsFriend("target", "player")) then
        texture:SetDesaturated(false)	
        dur:Show()		
      else
        texture:SetDesaturated(true)
		dur:Hide()
      end
	end
  end
end

local function updateReputationColor(self, event, unit, bar, min, max, value, name, id)
  bar:SetStatusBarColor(FACTION_BAR_COLORS[id].r, FACTION_BAR_COLORS[id].g, FACTION_BAR_COLORS[id].b)
end

local function styleFunc(self, unit)
  self.colors = colors
  self.menu = menu
  self:RegisterForClicks("AnyUp")
  self:SetAttribute("type2", "menu")
  self:SetScript("OnEnter", UnitFrame_OnEnter)
  self:SetScript("OnLeave", UnitFrame_OnLeave)

  self:SetBackdrop(backdrop)
  self:SetBackdropColor(0, 0, 0, 1)
  self:SetBackdropBorderColor(1, 1, 1, 1)

  self.Health = CreateFrame("StatusBar", nil, self)
  self.Health:SetPoint("TOPRIGHT", self)
  self.Health:SetPoint("TOPLEFT", self)
  self.Health:SetHeight(height - 4)
  self.Health:SetStatusBarTexture((unit == "player" or unit == "target") and texture or texture2)

  self.Health.Text = SetFontString(self.Health, fontn, 14, "THINOUTLINE")
  self.Health.Text:SetPoint("RIGHT", 0, -23)
  self.Health.Text:SetJustifyH("RIGHT")
  self.Health.Text.frequentUpdates = 0.1
  if self:GetParent():GetName():match'oUF_Party' then
    self:Tag(self.Health.Text, "[coreehpp]")
  elseif (unit == "player" or unit == "target") then
    self:Tag(self.Health.Text, "[coreehp]")
  end
  
  self.Health.Text2 = SetFontString(self.Health, fontn, (unit == "player" or unit == "target") and 20 or 14, "THINOUTLINE")
  self.Health.Text2.frequentUpdates = 0.1
  if(unit == "player") then
    self.Health.Text2:SetPoint("RIGHT", 40, -1)
  elseif(unit == "target") then
    self.Health.Text2:SetPoint("LEFT", -38, -1)
  elseif(unit == "targettarget" or unit == "pet") then
    self.Health.Text2:SetPoint("RIGHT", 0, -23)
  elseif(unit and unit:match("boss%d")) then
    self.Health.Text2:SetPoint("RIGHT",self.Health, 30, 1)  
  elseif(unit == "focus" or unit == "pet") then
    self.Health.Text2:SetPoint("LEFT",self.Health, -30, -1)
  else
    self.Health.Text2:SetPoint("LEFT",self.Health, -30, 0)
  end
  self:Tag(self.Health.Text2, "[coreehp2]")

  self.Health.bg = self.Health:CreateTexture(nil, "BORDER")
  self.Health.bg:SetAllPoints(self.Health)
  self.Health.bg:SetTexture((unit == "player" or unit == "target") and texture or texture2)
  self.Health.bg.multiplier = 0.3
  
  self.Health.colorTapping = true
  self.Health.colorDisconnected = true
  self.Health.colorClass = true
  self.Health.colorReaction = true
  
  self.Health.frequentUpdates = true
  self.Health.Smooth = true  

  self.Power = CreateFrame("StatusBar", nil, self)
  self.Power:SetPoint("TOPRIGHT", self.Health, "BOTTOMRIGHT", 0, -1)
  self.Power:SetPoint("TOPLEFT", self.Health, "BOTTOMLEFT", 0, -1)
  self.Power:SetStatusBarTexture(texture2)
  self.Power:SetHeight(3)

  self.Power.bg = self.Power:CreateTexture(nil, "BORDER")
  self.Power.bg:SetAllPoints(self.Power)
  self.Power.bg:SetTexture(texture2)
  self.Power.bg.multiplier = 0.3
  
  if (unit == "player" or unit == "target") then
    self.Power.Text = SetFontString(self.Health, fontn, 14, "THINOUTLINE")
    self.Power.Text:SetPoint("RIGHT", self.Health.Text, "LEFT")
    self.Power.Text.frequentUpdates = 0.1
    self:Tag(self.Power.Text, "[coreepp]")
  end
  
  self.Power.colorPower = true
  
  self.Power.frequentUpdates = true
  self.Power.Smooth = true

  local ricon = self.Health:CreateFontString(nil, "OVERLAY")
  ricon:SetPoint("TOP", self, 0, 8)
  ricon:SetJustifyH"LEFT"
  ricon:SetFontObject(GameFontNormalSmall)
  ricon:SetTextColor(1, 1, 1)
  self.RIcon = ricon
  self:RegisterEvent("RAID_TARGET_UPDATE", updateRIcon)
  table.insert(self.__elements, updateRIcon)    

  self.Info = SetFontString(self.Health, fontn, 14, "THINOUTLINE")
  self.Info:SetPoint("LEFT", self.Health, 1, -23)
  if(unit == "player") then
	self.Info:Hide()
	self:RegisterEvent("UNIT_ENTERED_VEHICLE", playerVehicle)
	self:RegisterEvent("UNIT_EXITED_VEHICLE", playerVehicle)		  
  elseif(unit == "targettarget" or unit == "pet") then    
    self.Info:SetPoint("RIGHT", self.Health.Text2, "LEFT")
  elseif(unit == "focus" or unit == "focustarget" or (unit and unit:match("boss%d"))) then    
    self.Info:SetPoint("LEFT", self.Health, 3, 0)
    self.Info:SetPoint("RIGHT", self.Health)
  elseif(self:GetParent():GetName():match'oUF_Party') then
    self.Info:SetPoint("LEFT", self.Health, 0, -23)
    self.Info:SetPoint("RIGHT", self.Health.Text, "LEFT")
  else  
    self.Info:SetPoint("RIGHT", self.Power.Text, "LEFT")
  end
  self:Tag(self.Info, unit == "target" and "[coreecolor][level][coreeclassi] |cFFFFFFFF[name]|r" or self:GetParent():GetName():match'oUF_Party' and "[coreeafk]|cFFFFFFFF[name]|r" or "|cFFFFFFFF[coreenamev]|r")
  
  if(unit == "player") then
    if(IsAddOnLoaded("oUF_Swing")) then      
      local pcolor = UnitIsPlayer("player") and RAID_CLASS_COLORS[select(2, UnitClass("player"))]
      self.Swing = CreateFrame("StatusBar", nil, self)
      self.Swing:SetPoint("BOTTOM", self, "TOP", 0, 14)
      self.Swing:SetStatusBarTexture(texture2)
      if(pcolor) then
        self.Swing:SetStatusBarColor(pcolor.r, pcolor.g, pcolor.b)
      else
        self.Swing:SetStatusBarColor(0.3, 0.5, .2)
      end      
      self.Swing:SetHeight(4)
      self.Swing:SetWidth(width)
      self.Swing:SetBackdrop(backdrop)
      self.Swing:SetBackdropColor(0, 0, 0)

      self.Swing.Text = self.Swing:CreateFontString(nil, "OVERLAY")
      self.Swing.Text:SetFont(fontn, 10, "THINOUTLINE")
      self.Swing.Text:SetPoint("BOTTOMRIGHT", self.Swing, "TOPRIGHT", 0,1)

      self.Swing.bg = self.Swing:CreateTexture(nil, "BORDER")
      self.Swing.bg:SetAllPoints(self.Swing)
      self.Swing.bg:SetTexture(0.4, 0.4, 0.4)        
    end
	if(IsAddOnLoaded("oUF_PowerSpark")) then
	  self.Spark = self.Power:CreateTexture(nil, "OVERLAY")
	  self.Spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	  self.Spark:SetVertexColor(1, 1, 1, 1)
	  self.Spark:SetBlendMode("ADD")
	  self.Spark:SetHeight(self.Power:GetHeight()*2)
	  self.Spark:SetWidth(4)
	  self.Spark = spark
	end
    if(class == "DRUID") then
      self.DruidPower = CreateFrame('StatusBar', nil, self)
      self.DruidPower:SetPoint('TOP', self.Health, 'BOTTOM')
      self.DruidPower:SetStatusBarTexture(texture)
      self.DruidPower:SetHeight(1)
      self.DruidPower:SetWidth(width)
      self.DruidPower:SetAlpha(0)

      local value = SetFontString(self.DruidPower, fontn, 14, "THINOUTLINE")
      value:SetPoint("LEFT",self.Health, 0, -23)
      self:Tag(value, '[druidpower]')

      table.insert(self.__elements, updateDruidPower)
      self:RegisterEvent('UNIT_MANA', updateDruidPower)
      self:RegisterEvent('UNIT_ENERGY', updateDruidPower)
      self:RegisterEvent('UPDATE_SHAPESHIFT_FORM', updateDruidPower)
    end
    if(select(2, UnitClass('player')) == 'DEATHKNIGHT') then
      self.Runes = CreateFrame('Frame', nil, self)
      self.Runes:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 0, -1)
      self.Runes:SetHeight(4)
      self.Runes:SetWidth(width)
      self.Runes.anchor = 'TOPLEFT'
      self.Runes.growth = 'RIGHT'
      self.Runes.height = 4
      self.Runes.width = width / 6 - 1
	  self.Runes.spacing = 1

      for index = 1, 6 do
        self.Runes[index] = CreateFrame('StatusBar', nil, self.Runes)
        self.Runes[index]:SetBackdrop(backdrop)
        self.Runes[index]:SetBackdropColor(0, 0, 0)
        self.Runes[index]:SetStatusBarTexture(texture2)

        self.Runes[index].bg = self.Runes[index]:CreateTexture(nil, 'BACKGROUND')
        self.Runes[index].bg:SetAllPoints(self.Runes[index])
        self.Runes[index].bg:SetTexture(texture2)
		self.Runes[index].bg.multiplier = 0.3
      end
    end   
    if IsAddOnLoaded("oUF_TotemBar") and class == "SHAMAN" then
	  self.TotemBar = {}
	  for i = 1, 4 do
		self.TotemBar[i] = CreateFrame("StatusBar", nil, self)
		self.TotemBar[i]:SetHeight(4)
		self.TotemBar[i]:SetWidth(width/4 - 0.85)
		if (i == 1) then
			self.TotemBar[i]:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -1)
		else
			self.TotemBar[i]:SetPoint("TOPLEFT", self.TotemBar[i-1], "TOPRIGHT", 1, 0)
		end
		self.TotemBar[i]:SetStatusBarTexture(texture2)
		self.TotemBar[i]:SetBackdrop(backdrop)
		self.TotemBar[i]:SetBackdropColor(0, 0, 0)
		self.TotemBar[i]:SetMinMaxValues(0, 1)
		self.TotemBar[i].destroy = true
						
		self.TotemBar[i].bg = self.TotemBar[i]:CreateTexture(nil, "BORDER")
		self.TotemBar[i].bg:SetAllPoints(self.TotemBar[i])
		self.TotemBar[i].bg:SetTexture(texture2)
		self.TotemBar[i].bg.multiplier = 0.3
	  end
    end	
    if(IsAddOnLoaded("oUF_Reputation")) then
      self.Reputation = CreateFrame("StatusBar", nil, self)
      if(IsAddOnLoaded("oUF_Experience") and UnitLevel("player") ~= MAX_PLAYER_LEVEL) then
        self.Reputation:SetPoint("BOTTOM", self, "TOP", 0, 7)
      else
        self.Reputation:SetPoint("BOTTOM",self,"TOP",0,2)
      end
      self.Reputation:SetHeight(4)
      self.Reputation:SetWidth(width)    
      self.Reputation:SetStatusBarTexture(texture2)
      self.Reputation:SetBackdrop(backdrop)
      self.Reputation:SetBackdropColor(0,0,0)  
      self.Reputation.PostUpdate = updateReputationColor
      
      self.Reputation.bg = self.Reputation:CreateTexture(nil, "BORDER")
      self.Reputation.bg:SetAllPoints(self.Reputation)
	  self.Reputation.bg:SetTexture(texture2)
	  self.Reputation.bg.multiplier = 0.3
      
      self.Reputation.Tooltip = true  
    end    
    if(UnitLevel("player") ~= MAX_PLAYER_LEVEL) then
      self.Resting = self.Health:CreateTexture(nil, "OVERLAY")
      self.Resting:SetHeight(16)
      self.Resting:SetWidth(16)
      self.Resting:SetPoint("TOPRIGHT", 0, 8)
    end  
	if inCombat then
	  self.Combat = self.Health:CreateTexture(nil, "OVERLAY")
      self.Combat:SetPoint("BOTTOMLEFT", self, -2, -8)
      self.Combat:SetHeight(20)
      self.Combat:SetWidth(20)
	end
  end
  if(IsAddOnLoaded("oUF_Experience") and (unit == "player" or unit == "pet")) then
    self.Experience = CreateFrame("StatusBar", nil, self)
    self.Experience:SetPoint("BOTTOM", self, "TOP", 0, 2)
    if(unit == "pet") then
      self.Experience:SetPoint("TOP",self,"BOTTOM",0,-1)
    end
    self.Experience:SetHeight(4)
    self.Experience:SetWidth((unit == "pet") and 100 or width)    
    self.Experience:SetStatusBarTexture(texture2)
    self.Experience:SetBackdrop(backdrop)
    self.Experience:SetBackdropColor(0,0,0)  
    self.Experience.PostUpdate = PostUpdateExperience    
    self:RegisterEvent("UPDATE_EXHAUSTION", PostUpdateExperience)
    
    self.Experience.bg = self.Experience:CreateTexture(nil, "BORDER")
    self.Experience.bg:SetAllPoints(self.Experience)
	self.Experience.bg:SetTexture(texture2)
	self.Experience.bg.multiplier = 0.3
    
    self.Experience.Tooltip = true      
  end
  if(cbar == true and (unit == "player" or unit == "target" or unit == "focus" or unit == "pet")) then
    do
      local cb = CreateFrame("StatusBar", nil, self)
      cb:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -21)
      cb:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -21)
      if (unit == "focus") then    
        cb:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -5)
        cb:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -5)
      end
      cb:SetStatusBarTexture(texture2, "OVERLAY")
      cb:SetStatusBarColor(0.2705882352941176, 0.407843137254902, 0.5450980392156862)
      cb:SetBackdrop(backdrop)
      cb:SetBackdropColor(0, 0, 0)
      cb:SetHeight(15)
      cb:SetWidth(300)
      cb:SetMinMaxValues(1, 100)
      cb:SetValue(1)
      cb:Hide()
      self.Castbar = cb  
    
      local cbbg = cb:CreateTexture(nil, "BORDER")
      cbbg:SetAllPoints(cb)
      cbbg:SetTexture(0.3, 0.3, 0.3)
      cb.bg = cbbg    

      local cbtime = SetFontString(cb, fontn, 14, "THINOUTLINE")
      cbtime:SetPoint("RIGHT", cb, -3, 1)
      cbtime:SetJustifyH("RIGHT")
      cb.CustomTimeText = OverrideCastbarTime
      cb.Time = cbtime

      local cbtext = SetFontString(cb, fontn, 14, "THINOUTLINE")
      cbtext:SetPoint("LEFT", cb, 3, 1)
      cbtext:SetPoint("RIGHT", cbtime, "LEFT")
      cb.Text = cbtext
    
      if(cbarsafe == true and unit == "player") then
        local cbsafe = cb:CreateTexture(nil,"ARTWORK")
        cbsafe:SetTexture(texture2)
        cbsafe:SetVertexColor(.69,.31,.31)
        cbsafe:SetPoint("TOPRIGHT")
        cbsafe:SetPoint("BOTTOMRIGHT")
        cb.SafeZone = cbsafe
      end    
	end
  end
 
  if (unit == "player" or self:GetParent():GetName():match'oUF_Party') then
    self.Leader = self.Health:CreateTexture(nil, "OVERLAY")
    self.Leader:SetPoint("TOPLEFT", self, 0, 8)
    self.Leader:SetHeight(16)
    self.Leader:SetWidth(16)
    
    self.Threat = self.Health:CreateTexture(nil, "OVERLAY")
    self.Threat:SetHeight(8)
    self.Threat:SetWidth(8)  
	self.Threat:SetTexture([=[Interface\AddOns\oUF_coree\media\indicator]=])
    self.Threat:SetPoint("TOPRIGHT", self.Health, 0, 0)  
  end
  if (unit == "target" or unit == "pet" or unit == "targettarget" or self:GetParent():GetName():match'oUF_Party') then
    self.Auras = CreateFrame("Frame", nil, self)
    self.Auras:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -1, 5)
    self.Auras.gap = true
    self.Auras.showDebuffType = true  
    self.Auras.spacing = 2
    self.Auras.initialAnchor = "BOTTOMLEFT"
    self.Auras["growth-y"] = "UP"      
    if(unit == "target") then
      self.CombatFeedbackText = self.Health:CreateFontString(nil, "OVERLAY", "GameFontNormal")
      self.CombatFeedbackText:SetPoint("CENTER", self)
      
      self.CPoints = SetFontString(self.Health, fontn, 16, "THINOUTLINE")
      self.CPoints:SetPoint("RIGHT", self.Health, -2, 0)
      self.CPoints.unit = "player"

      self.Auras:SetHeight(20)
      self.Auras:SetWidth(width)
      self.Auras.size = self.Auras:GetHeight()
	  
      self.PostUpdateAuraIcon = PostUpdateAuraIcon
    else
      self.BarFade = true
      self.BarFadeMinAlpha = 0.25
    end  
    if(unit == "pet") then
      self.Power.colorPower = true
      self.Power.colorHappiness = true
      self.Power.colorReaction = false
      
      self.Auras:SetHeight(self.Health:GetHeight())
      self.Auras:SetWidth(width - 106)
      self.Auras.size = self.Auras:GetHeight()

      self.CPoints = SetFontString(self.Health, fontn, 14, "THINOUTLINE")
      self.CPoints:SetPoint("LEFT", self.Health, 2, 0)
      self.CPoints.unit = "pet"
    end
    if(unit == "targettarget") then
      self.Auras:SetHeight(20)
      self.Auras:SetWidth(width - 106)
      self.Auras.size = self.Auras:GetHeight()
      self.Auras.num = 5
      self.Auras.numBuffs = 0
      self.Auras.numDebuffs = 5
      self.Auras.initialAnchor = "TOPRIGHT"
      self.Auras["growth-x"] = "LEFT"
    end    
    if (self:GetParent():GetName():match'oUF_Party') then  
      self:SetAttribute('initial-height', height)
      self:SetAttribute('initial-width', width - 86)
      
      self.Health.Text2:Hide()
      self.Auras:SetHeight(self.Health:GetHeight())
      self.Auras:SetWidth(width - 86)
      self.Auras.size = 18
      self.Auras.buffFilter = "HELPFUL|RAID"
      self.Auras.numBuffs = 3
      self.Auras.numDebuffs = 3
      
      self.outsideRangeAlpha = 0.4
      self.inRangeAlpha = 1.0
      self.Range = true  
      if(ppets and self:GetAttribute('unitsuffix') == 'pet') then
        self:SetAttribute('initial-height', height)
        self:SetAttribute('initial-width', width - 126)
        
        self.Health.Text:Hide()
        self.Health.colorReaction = true
        
        self.Power.colorPower = true
        
        self.Info:SetPoint("LEFT", self.Health, 0, -23)
        self.Info:SetPoint("RIGHT", self.Health)
        self.Threat:Hide()
        self.Leader:Hide()
        self.Auras:Hide()
      end
    end
  end
  if(IsAddOnLoaded("oUF_HealComm4")) then
    self.HealCommBar = CreateFrame('StatusBar', nil, self.Health)
	self.HealCommBar:SetHeight(0)
	self.HealCommBar:SetWidth(0)
	self.HealCommBar:SetStatusBarTexture(self.Health:GetStatusBarTexture():GetTexture())
	self.HealCommBar:SetStatusBarColor(0, 1, 0, 0.25)
	self.HealCommBar:SetPoint('LEFT', self.Health, 'LEFT')

	-- optional flag to show overhealing
	self.allowHealCommOverflow = true
  end

  if(not (unit == "target" or self:GetParent():GetName():match'oUF_Party')) then
    self.ignoreHealComm = true
  end  
  self.MoveableFrames = true
  --self.disallowVehicleSwap = true
  
  if(unit=="target") then
	self:RegisterEvent("UNIT_TARGET", function(self, event, unit)
	  if(unit=="target") then
        oUF.units.targettarget:PLAYER_ENTERING_WORLD'OnTargetUpdate'
  	  end
	end)
  end  
	
  self.PostCreateAuraIcon = createAura
  if(unit == "focustarget") then
    self.Health:SetHeight(13)
    self.Power:Hide()
    
    self:SetAttribute("initial-height", 13)
    self:SetAttribute('initial-width', width - 86)        
  elseif(unit == "focus" or (unit and unit:match("boss%d"))) then      
    self:SetAttribute('initial-height', height)
    self:SetAttribute('initial-width', width - 86)        
  elseif(unit == "pet" or unit == "targettarget") then
    self:SetAttribute('initial-height', height)
    self:SetAttribute('initial-width', width - 106)        
  elseif(unit == "player" or unit == "target") then
    self:SetAttribute('initial-height', height)
    self:SetAttribute('initial-width', width)
  end

  return self
end

oUF:RegisterStyle("coree", styleFunc)
oUF:SetActiveStyle("coree")
oUF:Spawn("player", "oUF_coree_player"):SetPoint("CENTER", UIParent, -220, -250)
oUF:Spawn("target", "oUF_coree_target"):SetPoint("CENTER", UIParent, 220, -250)

if IsAddOnLoaded("oUF_HealComm") then
  oUF:Spawn("targettarget", "oUF_coree_targett"):SetPoint("LEFT", "oUF_coree_target", "RIGHT", 7, -25)
else 
  oUF:Spawn("targettarget", "oUF_coree_targett"):SetPoint("LEFT", "oUF_coree_target", "RIGHT", 7, 0)
end
oUF:Spawn("pet", "oUF_coree_pet"):SetPoint("RIGHT", "oUF_coree_player", "LEFT", -7, 0)

oUF:Spawn("focus", "oUF_coree_focus"):SetPoint("RIGHT", UIParent, -50, 0)
oUF:Spawn("focustarget", "oUF_coree_focust"):SetPoint("TOP", "oUF_coree_focus", 0, 18)

local boss = {}
for i = 1, MAX_BOSS_FRAMES do
	boss[i] = oUF:Spawn("boss"..i, "oUF_Boss"..i)

	if i == 1 then
		boss[i]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 15, -215)
	else
		boss[i]:SetPoint("TOP", boss[i-1], "BOTTOM", 0, -5)
	end
end

for i, v in ipairs(boss) do v:Show() end

local party = oUF:Spawn("header", "oUF_Party")
party:SetPoint("TOPRIGHT","oUF_coree_pet", 0, 75)
party:SetManyAttributes("showParty", true, "yOffset", 83, "showPlayer", false)
if ppets then
  party:SetAttribute("template", "oUF_coreePPets")
end
if IsAddOnLoaded("oUF_MoveableFrames") then
  oUF_MoveableFrames_HEADER("oUF_Party", PartyAnchor, 0, 75)
end

local partyToggle = CreateFrame("Frame")
partyToggle:RegisterEvent("PLAYER_LOGIN")
partyToggle:RegisterEvent("RAID_ROSTER_UPDATE")
partyToggle:RegisterEvent("PARTY_LEADER_CHANGED")
partyToggle:RegisterEvent("PARTY_MEMBER_CHANGED")
partyToggle:SetScript("OnEvent", function(self)
  if(InCombatLockdown()) then
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
  else
    self:UnregisterEvent("PLAYER_REGEN_ENABLED")
    if (partyraid == true and GetNumRaidMembers() <=5 or GetNumRaidMembers() == 0) then
      party:Show()
    else
      party:Hide()
    end
  end
end)
